import * as Accessories from "./Accessories";
import HarmonyDataProvider from "./DataProviders/HarmonyDataProvider";
import * as Config from "./Models/Config";
import { IDevice } from "./Models";
import { HarmonyDevice } from "./Models/HarmonyDevice";

let Accessory: any;
let Homebridge: any;

/**
 * Main entry.
 * @param homebridge 
 */
export default function (homebridge: any) {
    Homebridge = homebridge;
    Accessory = homebridge.platformAccessory;
    homebridge.registerPlatform(
        'homebridge-harmony-watson',
        'HarmonyHubMatrix',
        HarmonyMatrixPlatform,
        true
    );
};

class HarmonyMatrixPlatform {
    log: any = {};
    config: Config.IConfig;
    api: any;
    dataProvider: HarmonyDataProvider | null;
    accessoryList: Array<Accessories.IAccessory> = [];

    constructor(log: any, config: any, api: any) {
        this.log = log;
        this.config = config;
        this.api = api;
        this.log('INFO - Registering Harmony Matrix Platform');
        this.api.on('didFinishLaunching', this.didFinishLaunching.bind(this));

        this.dataProvider = null;

        if (this.config) {
            //construct data provider
            this.dataProvider = new HarmonyDataProvider({
                hubs: this.config.Hubs,
                deviceConfigs: this.config.Devices,
                matrix: this.config.Matrix,
                log: this.log
            });

            //Emit devices if requested
            if (this.config.EmitDevicesOnStartup) {
                this.dataProvider.on("Ready", () => {

                    // TODO const hubs: { [hubName: string]: { [deviceName: string]: HarmonyDevice } } = this.dataProvider!.devicesByHub;
                    // Object.keys(hubs).forEach((hubName: string) => {
                    //     const deviceDictionary = hubs[hubName];
                    //     this.log(`${hubName}`)

                    //     Object.values(deviceDictionary).forEach((device: HarmonyDevice) => {
                    //         this.log(`    ${device.name} : ${device.id}`);
                    //         Object.keys(device.commands).forEach((command: string) => {
                    //             this.log(`        ${command}`);
                    //         });
                    //     });
                    // });

                });
            }
        }
    }

    /**
     * Handler for didFinishLaunching
     * Happens after constructor
     */
    didFinishLaunching() {
        this.log(`Publishing external accessories`);

        //This is required in order to have multiple tv remotes on one platform
        this.accessoryList.forEach((accessory: Accessories.IAccessory) => {
            if (accessory instanceof Accessories.ControlUnit) {
                this.api.publishExternalAccessories("HarmonyMatrixPlatform", [accessory.platformAccessory]);
            }
        })
    }

    /**
     * Called by homebridge to gather accessories.
     * @param callback 
     */
    accessories(callback: (accessories: Array<Accessories.IAccessory>) => void) {

        //Add control units
        this.config.ControlUnits.forEach((unit: Config.IControlUnit) => {
            this.accessoryList.push(new Accessories.ControlUnit({
                dataProvider: this.dataProvider!,
                displayName: unit.DisplayName,
                api: this.api,
                log: this.log,
                activities: unit.Activities,
                homebridge: Homebridge,
            }));
        });

        //Add device buttons
        this.config.DeviceButtons.forEach((button: Config.IDeviceButton) => {
            this.accessoryList.push(new Accessories.DeviceButton({
                dataProvider: this.dataProvider!,
                buttonName: button.ButtonName,
                displayName: button.DisplayName,
                deviceInfo: button,
                api: this.api,
                log: this.log,
                homebridge: Homebridge,

            }))
        });
        callback(this.accessoryList);
    }
}